/*
 * include/asm-microblaze/page.h -- VM ops
 *
 * Changes for MMU support:
 *    Copyright (C) 2007 Xilinx, Inc.  All rights reserved.
 *
 *  Copyright (C) 2003        John Williams <jwilliams@itee.uq.edu.au>
 *  Copyright (C) 2001, 2002  NEC Corporation
 *  Copyright (C) 2001, 2002  Miles Bader <miles@gnu.org>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of this
 * archive for more details.
 *
 * Written by Miles Bader <miles@gnu.org>
 * Microblaze port by John Williams
 */

#ifndef __MICROBLAZE_PAGE_H__
#define __MICROBLAZE_PAGE_H__

#include <asm/xparameters.h>
#include <linux/config.h>
#include <linux/pfn.h>


#define PAGE_SHIFT	12
#define PAGE_SIZE       (1UL << PAGE_SHIFT)
#define PAGE_MASK       (~(PAGE_SIZE-1))

#ifdef NO_MM

/*
 * PAGE_OFFSET -- the first address of the first page of memory. When not
 * using MMU this corresponds to the first free page in physical memory (aligned
 * on a page boundary).
 */
#ifndef PAGE_OFFSET
#define PAGE_OFFSET  (RAM_BASEADDR)
#endif

#else

/*
 * PAGE_OFFSET -- the first address of the first page of memory. With MMU
 * it is set to the kernel start address (aligned on a page boundary).
 *
 * CONFIG_KERNEL_START is defined in arch/microblaze/config.in and used
 * in arch/microblaze/Makefile.
 */
#define PAGE_OFFSET	CONFIG_KERNEL_START
#define KERNELBASE	PAGE_OFFSET

#endif /* NO_MM */

#define ARCH_PFN_OFFSET		(RAM_BASEADDR >> PAGE_SHIFT)

/*
 * MAP_NR -- given an address, calculate the index of the page struct which
 * points to the address's page.
 */
#define MAP_NR(addr) (((unsigned long)(addr) - PAGE_OFFSET) >> PAGE_SHIFT)

#ifdef __KERNEL__
#ifndef __ASSEMBLY__
/*
 * The basic type of a PTE - 32 bit physical addressing.
 */
typedef unsigned long pte_basic_t;
#define PTE_SHIFT	(PAGE_SHIFT - 2)	/* 1024 ptes per page */
#define PTE_FMT		"%.8lx"

#define STRICT_MM_TYPECHECKS

#define clear_page(page)		memset ((void *)(page), 0, PAGE_SIZE)
extern void copy_page(void *to, void *from);

#define clear_user_page(page, vaddr, pg) clear_page (page)
#define copy_user_page(to, from, vaddr, pg)	copy_page (to, from)

#define MB_MEMSTART	(unsigned long)(RAM_BASEADDR)
#define MB_MEMOFFSET	PAGE_OFFSET

#ifdef STRICT_MM_TYPECHECKS
/*
 * These are used to make use of C type-checking..
 */

typedef struct { unsigned long pte; } pte_t;
typedef struct { unsigned long pmd; } pmd_t;
typedef struct { unsigned long pgd; } pgd_t;
typedef struct { unsigned long pgprot; } pgprot_t;

#define pte_val(x)      ((x).pte)
#define pmd_val(x)      ((x).pmd)
#define pgd_val(x)      ((x).pgd)
#define pgprot_val(x)   ((x).pgprot)

#define __pte(x)        ((pte_t) { (x) } )
#define __pmd(x)        ((pmd_t) { (x) } )
#define __pgd(x)        ((pgd_t) { (x) } )
#define __pgprot(x)     ((pgprot_t) { (x) } )

#else /* !STRICT_MM_TYPECHECKS */
/*
 * .. while these make it easier on the compiler
 */

typedef unsigned long pte_t;
typedef unsigned long pmd_t;
typedef unsigned long pgd_t;
typedef unsigned long pgprot_t;

#define pte_val(x)      (x)
#define pmd_val(x)      (x)
#define pgd_val(x)      (x)
#define pgprot_val(x)   (x)

#define __pte(x)        (x)
#define __pmd(x)        (x)
#define __pgd(x)        (x)
#define __pgprot(x)     (x)

#endif /* STRICT_MM_TYPECHECKS */

#endif /* !__ASSEMBLY__ */

/* to align the pointer to the (next) page boundary */
#define PAGE_ALIGN(addr)	(((addr) + PAGE_SIZE - 1) & PAGE_MASK)


#define __pa(x)			__virt_to_phys ((unsigned long)(x))
#define __va(x)			((void *)__phys_to_virt ((unsigned long)(x)))
#ifndef CONFIG_DISCONTIGMEM
#define phys_to_pfn(phys)	(PFN_DOWN(phys))
#define pfn_to_phys(pfn)	(PFN_PHYS(pfn))
//#define pfn_to_page(pfn)        (mem_map + (pfn))
//#define page_to_pfn(page)       ((unsigned long)((page) - mem_map))
#define pfn_valid(pfn)          ((pfn) < (max_mapnr + ARCH_PFN_OFFSET))
#define virt_to_pfn(vaddr)	(phys_to_pfn((__pa(vaddr))))
#define pfn_to_virt(pfn)	__va(pfn_to_phys((pfn)))
#define	virt_addr_valid(vaddr)	(pfn_valid(virt_to_pfn(vaddr)))
#endif

/* Handle MicroBlaze processor with virtual memory.  */
#ifdef NO_MM
#define __virt_to_phys(addr)	(addr)
#define __phys_to_virt(addr)	(addr)
#else
#define __virt_to_phys(addr)	((addr) + RAM_BASEADDR - KERNELBASE)
#define __phys_to_virt(addr)	((addr) + KERNELBASE - RAM_BASEADDR)
#endif /* NO_MM */

#define virt_to_page(kaddr) 	(mem_map +  MAP_NR (kaddr))
#define VALID_PAGE(page) 	((page - mem_map) < max_mapnr)

#endif /* KERNEL */

#ifdef CONFIG_CONTIGUOUS_PAGE_ALLOC
#define WANT_PAGE_VIRTUAL 1 /* page alloc 2 relies on this */
#endif

#define VM_DATA_DEFAULT_FLAGS	(VM_READ | VM_WRITE | VM_EXEC | \
				 VM_MAYREAD | VM_MAYWRITE | VM_MAYEXEC)


#include <asm-generic/memory_model.h>
#include <asm-generic/page.h>

#endif /* __MICROBLAZE_PAGE_H__ */
